@@ -10,11 +10,11 @@
 #include <limits>
 #include <math.h>
 
-LUAU_FASTFLAG(LuauStoreCSTData)
-LUAU_FASTFLAG(LuauExtendStatEndPosWithSemicolon)
+LUAU_FASTFLAG(LuauStoreCSTData2)
 LUAU_FASTFLAG(LuauAstTypeGroup3)
 LUAU_FASTFLAG(LuauFixDoBlockEndLocation)
-LUAU_FASTFLAG(LuauParseOptionalAsNode)
+LUAU_FASTFLAG(LuauParseOptionalAsNode2)
+LUAU_FASTFLAG(LuauFixFunctionWithAttributesStartLocation)
 
 namespace
 {
@@ -167,7 +167,7 @@ struct StringWriter : Writer
 
     void symbol(std::string_view s) override
     {
-        if (FFlag::LuauStoreCSTData)
+        if (FFlag::LuauStoreCSTData2)
         {
             write(s);
         }
@@ -257,7 +257,7 @@ public:
             first = !first;
         else
         {
-            if (FFlag::LuauStoreCSTData && commaPosition)
+            if (FFlag::LuauStoreCSTData2 && commaPosition)
             {
                 writer.advance(*commaPosition);
                 commaPosition++;
@@ -1229,9 +1229,18 @@ struct Printer_DEPRECATED
                 AstType* l = a->types.data[0];
                 AstType* r = a->types.data[1];
 
-                auto lta = l->as<AstTypeReference>();
-                if (lta && lta->name == "nil")
-                    std::swap(l, r);
+                if (FFlag::LuauParseOptionalAsNode2)
+                {
+                    auto lta = l->as<AstTypeReference>();
+                    if (lta && lta->name == "nil" && !r->is<AstTypeOptional>())
+                        std::swap(l, r);
+                }
+                else
+                {
+                    auto lta = l->as<AstTypeReference>();
+                    if (lta && lta->name == "nil")
+                        std::swap(l, r);
+                }
 
                 // it's still possible that we had a (T | U) or (T | nil) and not (nil | T)
                 auto rta = r->as<AstTypeReference>();
@@ -1254,7 +1263,7 @@ struct Printer_DEPRECATED
 
             for (size_t i = 0; i < a->types.size; ++i)
             {
-                if (FFlag::LuauParseOptionalAsNode)
+                if (FFlag::LuauParseOptionalAsNode2)
                 {
                     if (a->types.data[i]->is<AstTypeOptional>())
                     {
@@ -1489,7 +1498,8 @@ struct Printer
 
     void visualize(AstExpr& expr)
     {
-        advance(expr.location.begin);
+        if (!expr.is<AstExprFunction>() || FFlag::LuauFixFunctionWithAttributesStartLocation)
+            advance(expr.location.begin);
 
         if (const auto& a = expr.as<AstExprGroup>())
         {
@@ -1623,6 +1633,17 @@ struct Printer
         }
         else if (const auto& a = expr.as<AstExprFunction>())
         {
+            for (const auto& attribute : a->attributes)
+                visualizeAttribute(*attribute);
+            if (FFlag::LuauFixFunctionWithAttributesStartLocation)
+            {
+                if (const auto cstNode = lookupCstNode<CstExprFunction>(a))
+                    advance(cstNode->functionKeywordPosition);
+            }
+            else
+            {
+                advance(a->location.begin);
+            }
             writer.keyword("function");
             visualizeFunctionBody(*a);
         }
@@ -1874,7 +1895,8 @@ struct Printer
 
     void visualize(AstStat& program)
     {
-        advance(program.location.begin);
+        if ((!program.is<AstStatLocalFunction>() && !program.is<AstStatFunction>()) || FFlag::LuauFixFunctionWithAttributesStartLocation)
+            advance(program.location.begin);
 
         if (const auto& block = program.as<AstStatBlock>())
         {
@@ -2111,13 +2133,36 @@ struct Printer
         }
         else if (const auto& a = program.as<AstStatFunction>())
         {
+            for (const auto& attribute : a->func->attributes)
+                visualizeAttribute(*attribute);
+            if (FFlag::LuauFixFunctionWithAttributesStartLocation)
+            {
+                if (const auto cstNode = lookupCstNode<CstStatFunction>(a))
+                    advance(cstNode->functionKeywordPosition);
+            }
+            else
+            {
+                advance(a->location.begin);
+            }
             writer.keyword("function");
             visualize(*a->name);
             visualizeFunctionBody(*a->func);
         }
         else if (const auto& a = program.as<AstStatLocalFunction>())
         {
+            for (const auto& attribute : a->func->attributes)
+                visualizeAttribute(*attribute);
+
             const auto cstNode = lookupCstNode<CstStatLocalFunction>(a);
+            if (FFlag::LuauFixFunctionWithAttributesStartLocation)
+            {
+                if (cstNode)
+                    advance(cstNode->localKeywordPosition);
+            }
+            else
+            {
+                advance(a->location.begin);
+            }
 
             writer.keyword("local");
 
@@ -2261,7 +2306,7 @@ struct Printer
 
         if (program.hasSemicolon)
         {
-            if (FFlag::LuauStoreCSTData)
+            if (FFlag::LuauStoreCSTData2)
                 advanceBefore(program.location.end, 1);
             writer.symbol(";");
         }
@@ -2271,7 +2316,7 @@ struct Printer
     {
         const auto cstNode = lookupCstNode<CstExprFunction>(&func);
 
-        // TODO(CLI-139347): need to handle attributes, argument types, and return type (incl. parentheses of return type)
+        // TODO(CLI-139347): need to handle return type (incl. parentheses of return type)
 
         if (func.generics.size > 0 || func.genericPacks.size > 0)
         {
@@ -2427,6 +2472,23 @@ struct Printer
         }
     }
 
+    void visualizeAttribute(AstAttr& attribute)
+    {
+        advance(attribute.location.begin);
+        switch (attribute.type)
+        {
+        case AstAttr::Checked:
+            writer.keyword("@checked");
+            break;
+        case AstAttr::Native:
+            writer.keyword("@native");
+            break;
+        case AstAttr::Deprecated:
+            writer.keyword("@deprecated");
+            break;
+        }
+    }
+
     void visualizeTypeAnnotation(AstType& typeAnnotation)
     {
         advance(typeAnnotation.location.begin);
@@ -2671,14 +2733,25 @@ struct Printer
         }
         else if (const auto& a = typeAnnotation.as<AstTypeUnion>())
         {
-            if (a->types.size == 2)
+            const auto cstNode = lookupCstNode<CstTypeUnion>(a);
+
+            if (!cstNode && a->types.size == 2)
             {
                 AstType* l = a->types.data[0];
                 AstType* r = a->types.data[1];
 
-                auto lta = l->as<AstTypeReference>();
-                if (lta && lta->name == "nil")
-                    std::swap(l, r);
+                if (FFlag::LuauParseOptionalAsNode2)
+                {
+                    auto lta = l->as<AstTypeReference>();
+                    if (lta && lta->name == "nil" && !r->is<AstTypeOptional>())
+                        std::swap(l, r);
+                }
+                else
+                {
+                    auto lta = l->as<AstTypeReference>();
+                    if (lta && lta->name == "nil")
+                        std::swap(l, r);
+                }
 
                 // it's still possible that we had a (T | U) or (T | nil) and not (nil | T)
                 auto rta = r->as<AstTypeReference>();
@@ -2699,12 +2772,20 @@ struct Printer
                 }
             }
 
+            if (cstNode && cstNode->leadingPosition)
+            {
+                advance(*cstNode->leadingPosition);
+                writer.symbol("|");
+            }
+
+            size_t separatorIndex = 0;
             for (size_t i = 0; i < a->types.size; ++i)
             {
-                if (FFlag::LuauParseOptionalAsNode)
+                if (FFlag::LuauParseOptionalAsNode2)
                 {
-                    if (a->types.data[i]->is<AstTypeOptional>())
+                    if (const auto optional = a->types.data[i]->as<AstTypeOptional>())
                     {
+                        advance(optional->location.begin);
                         writer.symbol("?");
                         continue;
                     }
@@ -2712,11 +2793,18 @@ struct Printer
 
                 if (i > 0)
                 {
-                    writer.maybeSpace(a->types.data[i]->location.begin, 2);
+                    if (cstNode && FFlag::LuauParseOptionalAsNode2)
+                    {
+                        // separatorIndex is only valid if `?` is handled as an AstTypeOptional
+                        advance(cstNode->separatorPositions.data[separatorIndex]);
+                        separatorIndex++;
+                    }
+                    else
+                        writer.maybeSpace(a->types.data[i]->location.begin, 2);
                     writer.symbol("|");
                 }
 
-                bool wrap = a->types.data[i]->as<AstTypeIntersection>() || a->types.data[i]->as<AstTypeFunction>();
+                bool wrap = !cstNode && (a->types.data[i]->as<AstTypeIntersection>() || a->types.data[i]->as<AstTypeFunction>());
 
                 if (wrap)
                     writer.symbol("(");
@@ -2729,15 +2817,27 @@ struct Printer
         }
         else if (const auto& a = typeAnnotation.as<AstTypeIntersection>())
         {
+            const auto cstNode = lookupCstNode<CstTypeIntersection>(a);
+
+            // If the sizes are equal, we know there is a leading & token
+            if (cstNode && cstNode->leadingPosition)
+            {
+                advance(*cstNode->leadingPosition);
+                writer.symbol("&");
+            }
+
             for (size_t i = 0; i < a->types.size; ++i)
             {
                 if (i > 0)
                 {
-                    writer.maybeSpace(a->types.data[i]->location.begin, 2);
+                    if (cstNode)
+                        advance(cstNode->separatorPositions.data[i - 1]);
+                    else
+                        writer.maybeSpace(a->types.data[i]->location.begin, 2);
                     writer.symbol("&");
                 }
 
-                bool wrap = a->types.data[i]->as<AstTypeUnion>() || a->types.data[i]->as<AstTypeFunction>();
+                bool wrap = !cstNode && (a->types.data[i]->as<AstTypeUnion>() || a->types.data[i]->as<AstTypeFunction>());
 
                 if (wrap)
                     writer.symbol("(");
@@ -2786,7 +2886,7 @@ std::string toString(AstNode* node)
     StringWriter writer;
     writer.pos = node->location.begin;
 
-    if (FFlag::LuauStoreCSTData)
+    if (FFlag::LuauStoreCSTData2)
     {
         Printer printer(writer, CstNodeMap{nullptr});
         printer.writeTypes = true;
@@ -2822,7 +2922,7 @@ void dump(AstNode* node)
 std::string transpile(AstStatBlock& block, const CstNodeMap& cstNodeMap)
 {
     StringWriter writer;
-    if (FFlag::LuauStoreCSTData)
+    if (FFlag::LuauStoreCSTData2)
     {
         Printer(writer, cstNodeMap).visualizeBlock(block);
     }
@@ -2836,7 +2936,7 @@ std::string transpile(AstStatBlock& block, const CstNodeMap& cstNodeMap)
 std::string transpileWithTypes(AstStatBlock& block, const CstNodeMap& cstNodeMap)
 {
     StringWriter writer;
-    if (FFlag::LuauStoreCSTData)
+    if (FFlag::LuauStoreCSTData2)
     {
         Printer printer(writer, cstNodeMap);
         printer.writeTypes = true;
