@@ -488,11 +488,12 @@ FreeType::FreeType(TypeLevel level, TypeId lowerBound, TypeId upperBound)
 {
 }
 
-FreeType::FreeType(Scope* scope, TypeId lowerBound, TypeId upperBound)
+FreeType::FreeType(Scope* scope, TypeId lowerBound, TypeId upperBound, Polarity polarity)
     : index(Unifiable::freshIndex())
     , scope(scope)
     , lowerBound(lowerBound)
     , upperBound(upperBound)
+    , polarity(polarity)
 {
 }
 
@@ -543,16 +544,18 @@ GenericType::GenericType(TypeLevel level)
 {
 }
 
-GenericType::GenericType(const Name& name)
+GenericType::GenericType(const Name& name, Polarity polarity)
     : index(Unifiable::freshIndex())
     , name(name)
     , explicitName(true)
+    , polarity(polarity)
 {
 }
 
-GenericType::GenericType(Scope* scope)
+GenericType::GenericType(Scope* scope, Polarity polarity)
     : index(Unifiable::freshIndex())
     , scope(scope)
+    , polarity(polarity)
 {
 }
 
@@ -630,23 +633,6 @@ FunctionType::FunctionType(TypeLevel level, TypePackId argTypes, TypePackId retT
 {
 }
 
-FunctionType::FunctionType(
-    TypeLevel level,
-    Scope* scope,
-    TypePackId argTypes,
-    TypePackId retTypes,
-    std::optional<FunctionDefinition> defn,
-    bool hasSelf
-)
-    : definition(std::move(defn))
-    , level(level)
-    , scope(scope)
-    , argTypes(argTypes)
-    , retTypes(retTypes)
-    , hasSelf(hasSelf)
-{
-}
-
 FunctionType::FunctionType(
     std::vector<TypeId> generics,
     std::vector<TypePackId> genericPacks,
@@ -683,27 +669,6 @@ FunctionType::FunctionType(
 {
 }
 
-FunctionType::FunctionType(
-    TypeLevel level,
-    Scope* scope,
-    std::vector<TypeId> generics,
-    std::vector<TypePackId> genericPacks,
-    TypePackId argTypes,
-    TypePackId retTypes,
-    std::optional<FunctionDefinition> defn,
-    bool hasSelf
-)
-    : definition(std::move(defn))
-    , generics(generics)
-    , genericPacks(genericPacks)
-    , level(level)
-    , scope(scope)
-    , argTypes(argTypes)
-    , retTypes(retTypes)
-    , hasSelf(hasSelf)
-{
-}
-
 Property::Property() {}
 
 Property::Property(
@@ -1306,9 +1271,9 @@ IntersectionTypeIterator end(const IntersectionType* itv)
     return IntersectionTypeIterator{};
 }
 
-TypeId freshType(NotNull<TypeArena> arena, NotNull<BuiltinTypes> builtinTypes, Scope* scope)
+TypeId freshType(NotNull<TypeArena> arena, NotNull<BuiltinTypes> builtinTypes, Scope* scope, Polarity polarity)
 {
-    return arena->addType(FreeType{scope, builtinTypes->neverType, builtinTypes->unknownType});
+    return arena->addType(FreeType{scope, builtinTypes->neverType, builtinTypes->unknownType, polarity});
 }
 
 std::vector<TypeId> filterMap(TypeId type, TypeIdPredicate predicate)
