@@ -3,9 +3,24 @@
 
 #include "Luau/Common.h"
 
+LUAU_FASTFLAG(LuauDeprecatedAttribute);
+
 namespace Luau
 {
 
+static bool hasAttributeInArray(const AstArray<AstAttr*> attributes, AstAttr::Type attributeType)
+{
+    LUAU_ASSERT(FFlag::LuauDeprecatedAttribute);
+
+    for (const auto attribute : attributes)
+    {
+        if (attribute->type == attributeType)
+            return true;
+    }
+
+    return false;
+}
+
 static void visitTypeList(AstVisitor* visitor, const AstTypeList& list)
 {
     for (AstType* ty : list.types)
@@ -277,6 +292,13 @@ bool AstExprFunction::hasNativeAttribute() const
     return false;
 }
 
+bool AstExprFunction::hasAttribute(const AstAttr::Type attributeType) const
+{
+    LUAU_ASSERT(FFlag::LuauDeprecatedAttribute);
+
+    return hasAttributeInArray(attributes, attributeType);
+}
+
 AstExprTable::AstExprTable(const Location& location, const AstArray<Item>& items)
     : AstExpr(ClassIndex(), location)
     , items(items)
@@ -791,13 +813,15 @@ AstStatTypeFunction::AstStatTypeFunction(
     const AstName& name,
     const Location& nameLocation,
     AstExprFunction* body,
-    bool exported
+    bool exported,
+    bool hasErrors
 )
     : AstStat(ClassIndex(), location)
     , name(name)
     , nameLocation(nameLocation)
     , body(body)
     , exported(exported)
+    , hasErrors(hasErrors)
 {
 }
 
@@ -894,6 +918,13 @@ bool AstStatDeclareFunction::isCheckedFunction() const
     return false;
 }
 
+bool AstStatDeclareFunction::hasAttribute(AstAttr::Type attributeType) const
+{
+    LUAU_ASSERT(FFlag::LuauDeprecatedAttribute);
+
+    return hasAttributeInArray(attributes, attributeType);
+}
+
 AstStatDeclareClass::AstStatDeclareClass(
     const Location& location,
     const AstName& name,
@@ -1057,6 +1088,13 @@ bool AstTypeFunction::isCheckedFunction() const
     return false;
 }
 
+bool AstTypeFunction::hasAttribute(AstAttr::Type attributeType) const
+{
+    LUAU_ASSERT(FFlag::LuauDeprecatedAttribute);
+
+    return hasAttributeInArray(attributes, attributeType);
+}
+
 AstTypeTypeof::AstTypeTypeof(const Location& location, AstExpr* expr)
     : AstType(ClassIndex(), location)
     , expr(expr)
