@@ -17,9 +17,6 @@
 
 #include <string.h>
 
-LUAU_DYNAMIC_FASTFLAGVARIABLE(LuauStackLimit, false)
-LUAU_DYNAMIC_FASTFLAGVARIABLE(LuauPopIncompleteCi, false)
-
 // keep max stack allocation request under 1GB
 #define MAX_STACK_SIZE (int(1024 / sizeof(TValue)) * 1024 * 1024)
 
@@ -183,10 +180,10 @@ static void correctstack(lua_State* L, TValue* oldstack)
 void luaD_reallocstack(lua_State* L, int newsize, int fornewci)
 {
     // throw 'out of memory' error because space for a custom error message cannot be guaranteed here
-    if (DFFlag::LuauStackLimit && newsize > MAX_STACK_SIZE)
+    if (newsize > MAX_STACK_SIZE)
     {
-        // reallocation was performaed to setup a new CallInfo frame, which we have to remove
-        if (DFFlag::LuauPopIncompleteCi && fornewci)
+        // reallocation was performed to setup a new CallInfo frame, which we have to remove
+        if (fornewci)
         {
             CallInfo* cip = L->ci - 1;
 
@@ -221,17 +218,7 @@ void luaD_reallocCI(lua_State* L, int newsize)
 
 void luaD_growstack(lua_State* L, int n)
 {
-    if (DFFlag::LuauPopIncompleteCi)
-    {
-        luaD_reallocstack(L, getgrownstacksize(L, n), 0);
-    }
-    else
-    {
-        if (n <= L->stacksize) // double size is enough?
-            luaD_reallocstack(L, 2 * L->stacksize, 0);
-        else
-            luaD_reallocstack(L, L->stacksize + n, 0);
-    }
+    luaD_reallocstack(L, getgrownstacksize(L, n), 0);
 }
 
 CallInfo* luaD_growCI(lua_State* L)
