@@ -17,7 +17,6 @@ LUAU_FASTFLAG(LuauSolverV2);
 LUAU_FASTFLAG(DebugLuauFreezeArena)
 LUAU_FASTFLAG(DebugLuauMagicTypes)
 LUAU_FASTFLAG(LuauSelectivelyRetainDFGArena)
-LUAU_FASTFLAG(LuauModuleHoldsAstRoot)
 LUAU_FASTFLAG(LuauImproveTypePathsInErrors)
 
 namespace
@@ -1555,7 +1554,6 @@ TEST_CASE_FIXTURE(FrontendFixture, "check_module_references_allocator")
 
 TEST_CASE_FIXTURE(FrontendFixture, "check_module_references_correct_ast_root")
 {
-    ScopedFastFlag sff{FFlag::LuauModuleHoldsAstRoot, true};
     fileResolver.source["game/workspace/MyScript"] = R"(
         print("Hello World")
     )";
@@ -1793,4 +1791,96 @@ TEST_CASE_FIXTURE(FrontendFixture, "test_invalid_dependency_tracking_per_module_
     CHECK(frontend.allModuleDependenciesValid("game/Gui/Modules/A", opts.forAutocomplete));
 }
 
+TEST_CASE_FIXTURE(FrontendFixture, "queue_check_simple")
+{
+    fileResolver.source["game/Gui/Modules/A"] = R"(
+        --!strict
+        return {hello=5, world=true}
+    )";
+    fileResolver.source["game/Gui/Modules/B"] = R"(
+        --!strict
+        local Modules = game:GetService('Gui').Modules
+        local A = require(Modules.A)
+        return {b_value = A.hello}
+    )";
+
+    frontend.queueModuleCheck("game/Gui/Modules/B");
+    frontend.checkQueuedModules();
+
+    auto result = frontend.getCheckResult("game/Gui/Modules/B", true);
+    REQUIRE(result);
+    LUAU_REQUIRE_NO_ERRORS(*result);
+}
+
+TEST_CASE_FIXTURE(FrontendFixture, "queue_check_cycle_instant")
+{
+    fileResolver.source["game/Gui/Modules/A"] = R"(
+        --!strict
+        local Modules = game:GetService('Gui').Modules
+        local B = require(Modules.B)
+        return {a_value = B.hello}
+    )";
+    fileResolver.source["game/Gui/Modules/B"] = R"(
+        --!strict
+        local Modules = game:GetService('Gui').Modules
+        local A = require(Modules.A)
+        return {b_value = A.hello}
+    )";
+
+    frontend.queueModuleCheck("game/Gui/Modules/B");
+    frontend.checkQueuedModules();
+
+    auto result = frontend.getCheckResult("game/Gui/Modules/B", true);
+    REQUIRE(result);
+    LUAU_REQUIRE_ERROR_COUNT(2, *result);
+    CHECK(toString(result->errors[0]) == "Cyclic module dependency: game/Gui/Modules/B -> game/Gui/Modules/A");
+    CHECK(toString(result->errors[1]) == "Cyclic module dependency: game/Gui/Modules/A -> game/Gui/Modules/B");
+}
+
+TEST_CASE_FIXTURE(FrontendFixture, "queue_check_cycle_delayed")
+{
+    fileResolver.source["game/Gui/Modules/C"] = R"(
+        --!strict
+        return {c_value = 5}
+    )";
+    fileResolver.source["game/Gui/Modules/A"] = R"(
+        --!strict
+        local Modules = game:GetService('Gui').Modules
+        local C = require(Modules.C)
+        local B = require(Modules.B)
+        return {a_value = B.hello + C.c_value}
+    )";
+    fileResolver.source["game/Gui/Modules/B"] = R"(
+        --!strict
+        local Modules = game:GetService('Gui').Modules
+        local C = require(Modules.C)
+        local A = require(Modules.A)
+        return {b_value = A.hello + C.c_value}
+    )";
+
+    frontend.queueModuleCheck("game/Gui/Modules/B");
+    frontend.checkQueuedModules();
+
+    auto result = frontend.getCheckResult("game/Gui/Modules/B", true);
+    REQUIRE(result);
+    LUAU_REQUIRE_ERROR_COUNT(2, *result);
+    CHECK(toString(result->errors[0]) == "Cyclic module dependency: game/Gui/Modules/B -> game/Gui/Modules/A");
+    CHECK(toString(result->errors[1]) == "Cyclic module dependency: game/Gui/Modules/A -> game/Gui/Modules/B");
+}
+
+TEST_CASE_FIXTURE(FrontendFixture, "queue_check_propagates_ice")
+{
+    ScopedFastFlag sffs{FFlag::DebugLuauMagicTypes, true};
+
+    ModuleName mm = fromString("MainModule");
+    fileResolver.source[mm] = R"(
+        --!strict
+        local a: _luau_ice = 55
+    )";
+    frontend.markDirty(mm);
+    frontend.queueModuleCheck("MainModule");
+
+    CHECK_THROWS_AS(frontend.checkQueuedModules(), InternalCompilerError);
+}
+
 TEST_SUITE_END();
