@@ -3,7 +3,7 @@
 
 #include "Luau/Common.h"
 
-#include <string.h>
+#include <vector>
 
 template<typename T>
 struct [[nodiscard]] ScopedFValue
@@ -49,4 +49,4 @@ public:
 };
 
 using ScopedFastFlag = ScopedFValue<bool>;
-using ScopedFastInt = ScopedFValue<int>;
+using ScopedFastInt = ScopedFValue<int>;
\ No newline at end of file
