@@ -556,10 +556,7 @@ TEST_CASE_FIXTURE(Fixture, "recursive_redefinition_reduces_rightfully")
 
 TEST_CASE_FIXTURE(BuiltinsFixture, "cli_142285_reduce_minted_union_func")
 {
-    ScopedFastFlag sffs[] = {
-        {FFlag::LuauSolverV2, true},
-        {FFlag::LuauDontForgetToReduceUnionFunc, true}
-    };
+    ScopedFastFlag sffs[] = {{FFlag::LuauSolverV2, true}, {FFlag::LuauDontForgetToReduceUnionFunc, true}};
 
     CheckResult result = check(R"(
         local function middle(a: number, b: number): number
@@ -582,7 +579,7 @@ TEST_CASE_FIXTURE(BuiltinsFixture, "cli_142285_reduce_minted_union_func")
     LUAU_REQUIRE_ERROR_COUNT(3, result);
     // There are three errors in the above snippet, but they should all be where
     // clause needed errors.
-    for (const auto& e: result.errors)
+    for (const auto& e : result.errors)
         CHECK(get<WhereClauseNeeded>(e));
 }
 
