@@ -30,6 +30,10 @@ LUAU_FASTFLAG(LuauInferLocalTypesInMultipleAssignments)
 LUAU_FASTFLAG(LuauUnifyMetatableWithAny)
 LUAU_FASTFLAG(LuauExtraFollows)
 LUAU_FASTFLAG(LuauImproveTypePathsInErrors)
+LUAU_FASTFLAG(LuauTypeCheckerAcceptNumberConcats)
+LUAU_FASTFLAG(LuauPreprocessTypestatedArgument)
+LUAU_FASTFLAG(LuauCacheInferencePerAstExpr)
+LUAU_FASTFLAG(LuauMagicFreezeCheckBlocked)
 
 using namespace Luau;
 
@@ -1862,7 +1866,6 @@ TEST_CASE_FIXTURE(BuiltinsFixture, "getmetatable_works_with_any")
             end,
         }
     )"));
-
 }
 
 TEST_CASE_FIXTURE(BuiltinsFixture, "getmetatable_infer_any_ret")
@@ -1937,14 +1940,98 @@ end
 TEST_CASE_FIXTURE(Fixture, "concat_string_with_string_union")
 {
     ScopedFastFlag _{FFlag::LuauSolverV2, true};
+    ScopedFastFlag fixNumberConcats{FFlag::LuauTypeCheckerAcceptNumberConcats, true};
+
     LUAU_REQUIRE_NO_ERRORS(check(R"(
-        local function foo(n : number): string return "" end
-        local function bar(n: number, m: string) end
-        local function concat_stuff(x, y)
-            local z = foo(x)
-            bar(y, z)
+        local function concat_stuff(x: string, y : string | number)
+            return x .. y
         end
     )"));
 }
 
+TEST_CASE_FIXTURE(BuiltinsFixture, "fuzz_local_before_declaration_ice")
+{
+    ScopedFastFlag sffs[] = {
+        {FFlag::LuauSolverV2, true},
+        {FFlag::LuauPreprocessTypestatedArgument, true},
+    };
+
+    CheckResult result = check(R"(
+        local _
+        table.freeze(_, _)
+    )");
+    LUAU_REQUIRE_ERROR_COUNT(2, result);
+    auto err0 = get<TypeMismatch>(result.errors[0]);
+    CHECK(err0);
+    CHECK_EQ("nil", toString(err0->givenType));
+    CHECK_EQ("table", toString(err0->wantedType));
+    auto err1 = get<CountMismatch>(result.errors[1]);
+    CHECK(err1);
+    CHECK_EQ(1, err1->expected);
+    CHECK_EQ(2, err1->actual);
+}
+
+TEST_CASE_FIXTURE(Fixture, "fuzz_dont_double_solve_compound_assignment" * doctest::timeout(1.0))
+{
+    ScopedFastFlag sffs[] = {{FFlag::LuauSolverV2, true}, {FFlag::LuauCacheInferencePerAstExpr, true}};
+
+    CheckResult result = check(R"(
+        local _ = {}
+        _[function<t0...>(...)
+            _[function(...)
+                _[_] %= _
+                _ = {}
+                _ = (- _)()
+            end] %= _
+            _[_] %= _
+        end] %= true
+    )");
+
+    LUAU_REQUIRE_ERRORS(result);
+    LUAU_REQUIRE_NO_ERROR(result, ConstraintSolvingIncompleteError);
+}
+
+TEST_CASE_FIXTURE(Fixture, "assert_allows_singleton_union_or_intersection")
+{
+    LUAU_REQUIRE_NO_ERRORS(check(R"(
+        local x = 42 :: | number
+        local y = 42 :: & number
+    )"));
+}
+
+TEST_CASE_FIXTURE(BuiltinsFixture, "assert_table_freeze_constraint_solving")
+{
+    ScopedFastFlag sffs[] = {
+        {FFlag::LuauSolverV2, true},
+        {FFlag::LuauMagicFreezeCheckBlocked, true}
+    };
+    LUAU_REQUIRE_NO_ERRORS(check(R"(
+        local f = table.freeze
+        f(table)
+    )"));
+}
+
+TEST_CASE_FIXTURE(BuiltinsFixture, "fuzz_assert_table_freeze_constraint_solving")
+{
+    ScopedFastFlag sffs[] = {
+        {FFlag::LuauSolverV2, true},
+        {FFlag::LuauMagicFreezeCheckBlocked, true}
+    };
+    // This is the original fuzzer version of the above issue.
+    CheckResult results = check(R"(
+        local function l0()
+        end
+        for l0 in false do
+        _ = (if _ then table)
+        repeat
+        do end
+        _:freeze(table)
+        until if _ then {{n0=_,},(_:freeze()._[_]),}
+        end
+    )");
+    LUAU_REQUIRE_ERRORS(results);
+    LUAU_REQUIRE_NO_ERROR(results, ConstraintSolvingIncompleteError);
+}
+
+
 TEST_SUITE_END();
